var workbenchState = {
  create: function() {
    game.add.sprite(0,0,"workbenchBackground");
    globalState.lastRoom = "workbench"
    
    if (globalState.currentWork.state == 'parcel'){
      // create the wip
      var work = game.add.sprite(400,300,'parcel');
      work.anchor.set(0.5)
      work.inputEnabled = true;
      work.events.onInputDown.addOnce(function() {
        // open parcel
        openParcel(work);
        setTools();
        setInventory();
      }, this);
      globalState.currentWork.wip = work;

    } else {
      continueWorking();
      setTools();
      setInventory();
    }
    addReturnToWorkshop();
  },
  update: function() {},
  render: function() {}
}

var center = {x:400,y:300}
var dropFinish = {x:697,y:105,w:102,h:393}
var dropWait = {x:0,y:105,w:102,h:393}
var tools = ["needle","eyes","ears","patches","bows"]
var toolbox;

function setInventory(){
  toolbox.tools = {};
  var workZone = {x:center.x-center.w/2,y:center.y-center.h/2,w:center.w,h:center.h}
  for (var i = 0; i < tools.length; i++) {
    var toolGroup = game.add.group();
    var tool = tools[i];
    toolGroup.visible = false;
    toolbox.addChild(toolGroup);
    toolbox.tools[tool] = toolGroup
    for (var j = globalState.userInventory[tool].length - 1; j >= 0; j--) {
      var item = tool+globalState.userInventory[tool][j];
      var x = game.rnd.integerInRange(toolbox.inside.x,toolbox.inside.x+toolbox.inside.w);
      var y = game.rnd.integerInRange(toolbox.inside.y,toolbox.inside.y+toolbox.inside.h);
      item = toolGroup.create(x,y,item);
      item.inventoryPosition = {tool:tool,index:j}
      item.anchor.set(0.5);
      item.inputEnabled = true;
      item.input.enableDrag();
      item.events.onDragStart.add(savePosition)
      item.events.onDragStop.add(function(sprite, pointer) {
        if (!inZone(sprite,toolbox.inside)){
          if (inZone(pointer,workZone)){
            setInWork(sprite,sprite.world)
          } else {
            snapBack(sprite);
          }
        }
      });
    };
  };
}

function setInWork(item,newPosition){
  item.parent.remove(item);
  globalState.currentWork.wip.addChild(item);
  item.x = newPosition.x-globalState.currentWork.wip.x;
  item.y = newPosition.y-globalState.currentWork.wip.y;
  // remove from inventory
  globalState.userInventory[item.inventoryPosition.tool].splice(item.inventoryPosition.index,1);
  item.inputEnabled = false;
}

function savePosition(item,pointer){
  item.x = pointer.x;
  item.y = pointer.y;
  item.originalPosition = {x:item.x,y:item.y}
}

function snapBack(item){
  item.x = item.originalPosition.x;
  item.y = item.originalPosition.y;
}

function setTools(){
  toolbox = game.add.image(-300,300,"toolbox");
  toolbox.anchor.set(0,0.5);
  toolbox.inside = {x:30,y:-150,w:150,h:300}
  var handle = game.add.button(225,0,"toolboxHandle",
    function(){
      closeToolbox()      
    },this,0,0,0);
  handle.anchor.set(0,0.5);
  toolbox.addChild(handle);

  for (var i = 0; i < tools.length; i++) {
    var tool = game.add.button(100+i*150,50,tools[i]+"Button",function(button){
      if (toolbox.isOpen){
        closeToolbox(function(){
          openToolbox(button.toolName);
        })
      } else {
        openToolbox(button.toolName);
      }
    },this,0,0,0);
    tool.toolName = tools[i];
    tool.anchor.set(0.5);      
  }
}

function openToolbox(tool,callbackOnComplete){
  toolbox.openTool = tool;
  toolbox.tools[tool].visible = true;
  var tween = game.add.tween(toolbox).to({x:-50},500).start();
  tween.onComplete.add(function(){
    toolbox.isOpen = true;
    if (callbackOnComplete){
      callbackOnComplete();
    }
  });
}

function closeToolbox(callbackOnComplete){
    var tween = game.add.tween(toolbox).to({x:-300},500).start();
    tween.onComplete.add(function(){
      toolbox.tools[toolbox.openTool].visible = false;
      toolbox.isOpen = false;
      if (callbackOnComplete){
        callbackOnComplete();
      }
    });
}

function addReturnToWorkshop(){
  var button = game.add.sprite(center.x,600,"backToWorkshop");
  button.anchor.set(0.5,1);
  button.inputEnabled = true;
  button.events.onInputDown.add(function(){
    globalState.currentWork.wip.scale.set(0.2);
    game.world.removeChild(globalState.currentWork.wip);
    game.state.start("workshop");
  })
}

function continueWorking(){
  var work = globalState.currentWork.wip;
  work.scale.set(1);
  work.x = center.x;
  work.y = center.y;
  center.w = work.width;
  center.h = work.height;
  work.events.onDragStop.removeAll();
  work.events.onInputDown.removeAll();
  game.add.existing(work);
  addWaitAndFinishEvents(work);
}

function openParcel(parcel){
  var work = game.add.sprite(parcel.x,parcel.y,globalState.currentWork.toy);
  work.anchor.set(0.5);
  work.originalPosition = {x:parcel.x,y:parcel.y}
  center.w = work.width;
  center.h = work.height;
  parcel.kill();
  globalState.currentWork.wip = work;
  globalState.currentWork.state = 'wip';
  work.inputEnabled = true;
  work.input.enableDrag();
  addWaitAndFinishEvents(work);
}

function addWaitAndFinishEvents(work){
  work.events.onDragStop.add(function(work, pointer) {
    if (inZone(work,dropFinish)) {
      work.kill();
      globalState.currentWork = null;
      game.state.start("workshop");
    } else if (inZone(work,dropWait)) {
      var waitIndex = nextFreeSpace(globalState.waiting);
      if (waitIndex!=-1){
        game.world.removeChild(work);
        globalState.waiting[waitIndex] = globalState.currentWork;
        globalState.currentWork = null;
        game.state.start("workshop");
      } else {
        snapBack(work);
      }
    } else {
      snapBack(work);
    }
  }, this);
}

function inZone(sprite,zone){
  return  sprite.x > zone.x && sprite.x < zone.x + zone.w &&
          sprite.y > zone.y && sprite.y < zone.y + zone.h
}