var workshopState = {
  create: function() {
    game.add.sprite(0, 0, "workshopBackground");
    game.add.sprite(256, 255, "table");
    game.add.sprite(572, 158, "shelf");
    setCurrentWork(true);
    setDoor();
    setWorks();
    setWaiting();
    setStatusFrame(31, 481);
    if (globalState.lastRoom == "store" || globalState.lastRoom === null) {
      player = createPlayer(800, 500, 1);
      enterWorkshop(player, () => {
        checkIfThereAreClients();
      });
    } else {
      player = createPlayer(390,350,2)
      checkIfThereAreClients();
    }
  },
  update: function() {},
  bubble: null
};

var dropzone = { x: 256, y: 255, w: 264, h: 109 };

function checkIfThereAreClients() {
  if (globalState.clients.length > 0) {
    if (!workshopState.bubble) {
      workshopState.bubble = game.add.sprite(740, 472, "newClientRingBubble");
      workshopState.bubble.anchor.set(0.5);
      workshopState.bubble.alpha = 0.5;
      game.add
        .tween(workshopState.bubble)
        .to({ alpha: 1 }, 1000, Phaser.Easing.Default , true, 0, -1, true)
        .loop();
    }
  } else {
    if (workshopState.bubble) {
      workshopState.bubble.kill();
      workshopState.bubble = null;
    }
  }
}


function setCurrentWork(addToWorld) {
  if (globalState.currentWork) {
    var currentWork = globalState.currentWork.wip;
    if (addToWorld){
        game.add.existing(currentWork);
    }
    currentWork.x = dropzone.x + dropzone.w / 2;
    currentWork.y = dropzone.y + dropzone.h / 2;
    currentWork.inputEnabled = true;
    currentWork.events.onDragStop.removeAll();
    currentWork.events.onInputDown.removeAll();
    currentWork.events.onInputDown.addOnce(function() {
      removeWips();
      game.state.start("workbench");
    }, this);
  }
}

function createToyBubble(x,y,toy){
    var bubble = game.add.sprite(0, -50, "speechBubble");
    bubble.anchor.set(0.5)
    var item = game.add.sprite(0,0,toy+"Icon");
    item.anchor.set(0.5);
    bubble.addChild(item);
    return bubble;
}

function setWaiting() {
    for (var i = globalState.waiting.length - 1; i >= 0; i--) {
        if (globalState.waiting[i]){
            var work = globalState.waiting[i].wip;
            work.workIndex = i;
            work.shelf = {x:100,y:400 + i*45}
            work.x = work.shelf.x;
            work.y = work.shelf.y;
            work.scale.set(0.2);
            work.events.onDragStop.removeAll();
            if (!globalState.currentWork) {
                work.events.onDragStop.add(function(sprite, pointer) {
                    if (inZone(sprite,dropzone)) {
                        // dropped
                        // workGroup.forEach(function(child) {
                        //   child.input.disableDrag();
                        // }, this);
                        globalState.currentWork = globalState.waiting[sprite.workIndex];
                        globalState.currentWork.state = "wip";
                        globalState.waiting[sprite.workIndex] = null;
                        setCurrentWork();
                    } else {
                        sprite.x = sprite.shelf.x;
                        sprite.y = sprite.shelf.y;
                    }
                }, this);
            }
            
            game.add.existing(work);
        }
    };
}

function setWorks() {
  var workGroup = game.add.group();
  for (var i = globalState.works.length - 1; i >= 0; i--) {
    if (globalState.works[i]) {
      var x = 620 + (i % 2) * 75;
      var y = 240 + Math.floor(i / 2) * 45;
      var work = workGroup.create(x, y, "parcelIcon");
      work.shelf = { x: x, y: y };
      work.anchor.set(0.5);
      work.workIndex = i;
      work.bubble = createToyBubble(0,-50,globalState.works[i].toy);
      work.bubble.alpha = 0;
      work.addChild(work.bubble);
      work.inputEnabled = true;
      work.events.onInputOver.add(function(sprite) {
        sprite.bubble.alpha = 1;
      }, this);
      work.events.onInputOut.add(function(sprite) {
        sprite.bubble.alpha = 0;
      }, this);

      if (!globalState.currentWork) {
        work.input.enableDrag();
        work.events.onDragStop.add(function(sprite, pointer) {
          if (inZone(sprite,dropzone)) {
            // dropped
            workGroup.forEach(function(child) {
              child.input.disableDrag();
            }, this);
            workGroup.removeChild(sprite)
            globalState.currentWork = globalState.works[sprite.workIndex];
            globalState.currentWork.state = "parcel";
            globalState.works[sprite.workIndex] = null;
            globalState.currentWork.wip = sprite;
            setCurrentWork(true);
          } else {
            sprite.x = sprite.shelf.x;
            sprite.y = sprite.shelf.y;
          }
        }, this);
      }
    }
  }
}

function setDoor() {
  var door = game.add.sprite(750, 450, "door");
  door.inputEnabled = true;
  door.events.onInputDown.add(function() {
    leaveWorkshop(player);
  }, this);
}

function enterWorkshop(player, callback) {
  player.animations.play("walk-left", 12, true);
  player.walking = true;
  var tween = game.add.tween(player);
  tween.to({ x: 390 }, 1000);
  tween.onComplete.addOnce(function() {
    player.animations.play("walk-up", 12, true);
    tween = game.add.tween(player).to({ y: 350 }, 600);
    tween.onComplete.addOnce(function() {
      player.frame = 2;
      player.animations.stop();
      player.walking = false;
      callback();
    }, this);
    tween.start();
  }, this);
  tween.start();
}

function leaveWorkshop(player) {
  if (player.walking) {
    return;
  }
  player.animations.play("walk-down", 12, true);
  var tween = game.add.tween(player);
  tween.to({ y: 500 }, 600);
  tween.onComplete.addOnce(function() {
    player.animations.play("walk-right", 12, true);
    tween = game.add.tween(player).to({ x: 800 }, 1000);
    tween.onComplete.addOnce(function() {
      workshopState.bubble = null;
      removeWips();
      game.state.start("store");
    }, this);
    tween.start();
  }, this);
  tween.start();
}

function removeWips(){
    for (var i = globalState.waiting.length - 1; i >= 0; i--) {
        if (globalState.waiting[i]){
            game.world.removeChild(globalState.waiting[i].wip);
        }
    };
    if (globalState.currentWork){
        game.world.removeChild(globalState.currentWork.wip);
    }
}